"""
Wiggelen command line interface.

.. moduleauthor:: Martijn Vermaat <martijn@vermaat.name>

.. Licensed under the MIT license, see the LICENSE file.
"""


import sys
import argparse

from .wiggle import fill, walk, write
from .index import index, write_index
from .merge import merge, mergers
from .distance import metrics, distance
from .transform import (backward_divided_difference,
                        forward_divided_difference,
                        central_divided_difference)
from . import intervals

# Python 3 compatibility.
try:
    from itertools import imap, ifilter
    map_ = imap
    filter_ = ifilter
except ImportError:
    map_ = map
    filter_ = filter

# Matplotlib only if it is installed.
try:
    from matplotlib import pyplot
except ImportError:
    pyplot = None


def log(message):
    sys.stderr.write(message + '\n')


def abort(message=None):
    if message:
        log('error: ' + message)
    sys.exit(1)


def index_track(track):
    """
    Build index for wiggle track.
    """
    # Todo: This will not rebuild the index if it already exists.
    idx, filename = index(track, force=True)
    if filename is None:
        abort('Could not write index file')


def sort_track(track):
    """
    Sort wiggle track regions alphabetically.
    """
    write(walk(track, force_index=True))


def scale_track(track, factor=0.1):
    """
    Scale values in a wiggle track.
    """
    scale = lambda (r, p, v): (r, p, v * factor)
    write(map_(scale, walk(track)))


def derivative_track(track, method='forward', step=None, auto_step=False):
    """
    Create derivative of a wiggle track.
    """
    kwargs = {'step': step}
    if method == 'central':
        derivative = central_divided_difference
    elif method == 'backward':
        derivative = backward_divided_difference
        kwargs['auto_step'] = auto_step
    else:
        derivative = forward_divided_difference
        kwargs['auto_step'] = auto_step
    write(derivative(walk(track), **kwargs))


def visualize_track(track):
    """
    Visualize a wiggle track.
    """
    # Todo: Only visualize one chromosome.
    pyplot.plot([v for _, _, v in fill(walk(track), filler=0)])
    pyplot.show()


def coverage_track(track, threshold=None):
    """
    Create coverage BED track of a wiggle track.
    """
    # Todo: Define coverage per region, like in `coverage-wiggle-to-bed` from
    #     bio-playground (https://github.com/martijnvermaat/bio-playground).
    walker = walk(track)
    if threshold is not None:
        walker = filter_(lambda (r, p, v): v >= threshold, walker)

    intervals.write(intervals.coverage(walker))


def merge_tracks(tracks, merger='sum', no_indices=False):
    """
    Merge any number of wiggle tracks in various ways.
    """
    walkers = [walk(track, force_index=not no_indices)
               for track in tracks]
    write(merge(*walkers, merger=mergers[merger]))


def distance_tracks(tracks, metric='a', threshold=None):
    """
    Calculate the distance between wiggle tracks.
    """
    # Todo: Cleanup this code.
    distances = distance(*tracks, metric=metrics[metric],
                          threshold=threshold)
    def name(index):
        return chr(ord('A') + index)
    try:
        sys.stdout.write(''.join('%s: %s\n' % (name(i), track.name)
                                 for i, track in enumerate(tracks)))
    except IOError:
        pass
    sys.stdout.write('\n   ')
    sys.stdout.write(' '.join('   %s ' % name(i)
                              for i in range(len(tracks))))
    sys.stdout.write('\n')
    sys.stdout.write(name(0) + '     x\n')
    for i in range(1, len(tracks)):
        sys.stdout.write('%s  ' % name(i))
        for j in range(0, i):
            sys.stdout.write(' %.3f' % distances[i, j])
        sys.stdout.write('   x\n')


def main():
    """
    Wiggelen command line interface.
    """
    parser = argparse.ArgumentParser(description=__doc__.split('\n\n')[0])
    subparsers = parser.add_subparsers(
        title='subcommands', dest='subcommand', help='subcommand help')

    p = subparsers.add_parser(
        'index', help='build index for wiggle track',
        description=index_track.__doc__.split('\n\n')[0])
    p.set_defaults(func=index_track)
    p.add_argument(
        'track', metavar='TRACK', type=argparse.FileType('r'),
        help='wiggle track')

    p = subparsers.add_parser(
        'sort', help='sort wiggle track regions alphabetically',
        description=sort_track.__doc__.split('\n\n')[0])
    p.set_defaults(func=sort_track)
    p.add_argument(
        'track', metavar='TRACK', type=argparse.FileType('r'),
        help='wiggle track')

    p = subparsers.add_parser(
        'scale', help='scale values in a wiggle track',
        description=scale_track.__doc__.split('\n\n')[0])
    p.set_defaults(func=scale_track)
    p.add_argument(
        'track', metavar='TRACK', type=argparse.FileType('r'),
        help='wiggle track')
    p.add_argument(
        '-f', '--factor', dest='factor', type=float, default=0.1,
        help='scaling factor to use (default: %(default)s)')

    p = subparsers.add_parser(
        'derivative', help='create derivative of a wiggle track',
        description=derivative_track.__doc__.split('\n\n')[0])
    p.set_defaults(func=derivative_track)
    p.add_argument(
        'track', metavar='TRACK', type=argparse.FileType('r'),
        help='wiggle track')
    p.add_argument(
        '-m', '--method', dest='method', type=str, default='forward',
        choices=('forward', 'backward', 'central'),
        help='type of divided difference method to use (default: %(default)s)')
    p.add_argument(
        '-s', '--step', dest='step', type=int, default=None,
        help='restrict to positions that are this far apart (default: no restriction)')
    p.add_argument(
        '-a', '--auto-step', dest='auto_step', action='store_true',
        help='automatically set STEP to a value based on the first two '
        'positions in TRACK (only used if STEP is omitted, always set if '
        'METHOD is central)')

    if pyplot is not None:
        p = subparsers.add_parser(
            'visualize', help='visualize a wiggle track (requires matplotlib)',
            description=visualize_track.__doc__.split('\n\n')[0])
        p.set_defaults(func=visualize_track)
        p.add_argument(
            'track', metavar='TRACK', type=argparse.FileType('r'),
            help='wiggle track')

    p = subparsers.add_parser(
        'coverage', help='create coverage BED track of a wiggle track',
        description=coverage_track.__doc__.split('\n\n')[0])
    p.set_defaults(func=coverage_track)
    p.add_argument(
        'track', metavar='TRACK', type=argparse.FileType('r'),
        help='wiggle track')
    p.add_argument(
        '-t', '--threshold', dest='threshold', type=int, default=None,
        help='only include positions with this value or higher (default: no '
        'threshold)')

    p = subparsers.add_parser(
        'merge', help='merge any number of wiggle tracks in various ways',
        description=merge_tracks.__doc__.split('\n\n')[0])
    p.set_defaults(func=merge_tracks)
    p.add_argument(
        '-m', dest='merger', choices=mergers, default='sum',
        help='merge operation to use (default: %(default)s)')
    p.add_argument(
        '-n', '--no-indices', dest='no_indices', action='store_true',
        help='assume tracks are sorted, don\'t force building indices')
    p.add_argument(
        'tracks', metavar='TRACK', nargs='+', type=argparse.FileType('r'),
        help='wiggle track')

    # Todo: Add additional information on the metrics (using the epilog
    # argument of the subparser).
    p = subparsers.add_parser(
        'distance', help='calculate the distance between wiggle tracks',
        description=distance_tracks.__doc__.split('\n\n')[0])
    p.set_defaults(func=distance_tracks)
    p.add_argument(
        '-m', dest='metric', choices=metrics, default='a',
        help='pairwise distance metric to use (default: %(default)s)')
    p.add_argument(
        '-t', dest='threshold', type=float, default=None,
        help='threshold for noise filter (default: no noise filter)')
    p.add_argument(
        'tracks', metavar='TRACK', nargs='+', type=argparse.FileType('r'),
        help='wiggle track')

    args = parser.parse_args()

    try:
        args.func(**{k: v for k, v in vars(args).items()
                     if k not in ('func', 'subcommand')})
    except IOError as e:
        abort(str(e))


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import sys
import matplotlib.pyplot as plt
import wiggelen


GENOME_SIZE = 16569
RESOLUTION = int(sys.argv[2]) if len(sys.argv) > 2 else 100

positions = []
values = []


def downscale(walker, resolution):
    previous_position = -resolution
    for chrom, position, value in walker:
        if position >= previous_position + resolution:
            previous_position = position
            yield chrom, position, value


for _, position, value in downscale(wiggelen.walk(open(sys.argv[1])), RESOLUTION):
    positions.append(position)
    values.append(value)


plt.plot(positions, values, linewidth=1.0)
plt.axis([1, GENOME_SIZE, min(values), max(values)])

plt.xlabel('mitochondrial genome')
plt.ylabel('value')
plt.title('Wiggle track')
#plt.grid(True)
plt.show()



"""
Helper functions for parsing wiggle tracks.

.. moduleauthor:: Martijn Vermaat <martijn@vermaat.name>

.. Licensed under the MIT license, see the LICENSE file.
"""


from collections import namedtuple


class ParseError(Exception):
    """
    Raised if a wiggle track cannot be parsed.
    """
    pass


# Line types.
class LineType(object):
    NONE, REGION, DATA = range(3)


# Track modes.
class Mode(object):
    VARIABLE, FIXED = range(2)


Data = namedtuple('Data', 'position span value')


# Create a new object encapsulating state for the parse function.
def create_state():
    return dict(mode=Mode.VARIABLE, span=1, start=None, step=None)


def parse(line, state):
    # Parse a line and return a tuple (line_type, data). The state dictionary is
    # modified and should be passed as such with the next call.
    if line.startswith('browser') or line.startswith('track') \
           or line.startswith('#') or line.isspace():
        # As far as I can see empty lines and comments are not allowed
        # by the spec, but I guess they exist in the real world.
        return LineType.NONE, None

    if line.startswith('variableStep'):
        try:
            fields = dict(map(lambda field: field.split('='),
                              line[len('variableStep'):].split()))
            state['mode'] = Mode.VARIABLE
            state['span'] = int(fields.get('span', 1))
            return LineType.REGION, fields['chrom']
        except (ValueError, KeyError):
            raise ParseError('Could not parse line: %s' % line)

    if line.startswith('fixedStep'):
        try:
            fields = dict(map(lambda field: field.split('='),
                              line[len('fixedStep'):].split()))
            state['mode'] = Mode.FIXED
            state['start'] = int(fields['start'])
            state['step'] = int(fields['step'])
            state['span'] = int(fields.get('span', 1))
            return LineType.REGION, fields['chrom']
        except (ValueError, KeyError):
            raise ParseError('Could not parse line: %s' % line)

    if state['mode'] == Mode.VARIABLE:
        try:
            position, value = line.split()
            return LineType.DATA, Data(
                position=int(position),
                span=state['span'],
                value=float(value) if '.' in value else int(value))
        except ValueError:
            raise ParseError('Could not parse line: %s' % line)

    if state['mode'] == Mode.FIXED:
        try:
            position = state['start']
            state['start'] += state['step']
            return LineType.DATA, Data(
                position=position,
                span=state['span'],
                value=float(line) if '.' in line else int(line))
        except ValueError:
            raise ParseError('Could not parse line: %s' % line)

    raise ParseError('Could not parse line: %s' % line)



"""
Wiggelen, working with wiggle tracks in Python.

The `wiggle`_ (WIG) format is for storing dense, continuous genomic data such
as GC percent, probability scores, read depth, and transcriptome data.

.. _wiggle: https://cgwb.nci.nih.gov/goldenPath/help/wiggle.html

.. moduleauthor:: Martijn Vermaat <martijn@vermaat.name>

.. Licensed under the MIT license, see the LICENSE file.
"""


from .parse import ParseError
from .wiggle import ReadError, walk, zip_, fill, write


# We follow a versioning scheme compatible with setuptools [1] where the
# package version is always that of the upcoming release (and not that of the
# previous release), post-fixed with ``.dev``. Only in a release commit, the
# ``.dev`` is removed (and added again in the next commit).
#
# Note that this scheme is not 100% compatible with SemVer [2] which would
# require ``-dev`` instead of ``.dev``.
#
# [1] http://peak.telecommunity.com/DevCenter/setuptools#specifying-your-project-s-version
# [2] http://semver.org/


__version_info__ = ('0', '1', '4')
__date__ = '15 Jul 2013'


__version__ = '.'.join(__version_info__)
__author__ = 'LUMC, Martijn Vermaat, Jeroen Laros'
__contact__ = 'm.vermaat.hg@lumc.nl'
__homepage__ = 'https://github.com/martijnvermaat/wiggelen'



"""
Read and write wiggle tracks.

.. todo:: Now that we also include the end positions in the index, it is
    possible to do random jumps inside a region with some educated guessing.
    Perfect hits would not be possible, since the length of the lines is
    variable.

.. moduleauthor:: Martijn Vermaat <martijn@vermaat.name>

.. Licensed under the MIT license, see the LICENSE file.
"""


import sys

from .parse import LineType, create_state, parse
from .index import ReadError, index, write_index


def walk(track=sys.stdin, force_index=False):
    """
    Walk over the track and yield (region, position, value) tuples.

    The values are always of type `int` or `float`.

    :arg track: Wiggle track.
    :type track: file
    :arg force_index: Force creating an index if it does not yet exist.
    :type force_index: bool

    :return: Tuples of (region, position, value) per defined position.
    :rtype: generator(str, int, _)

    Example::

        >>> for x in walk():
        ...     x
        ...
        ('chr18', 34344, 629.0)
        ('chr18', 34345, 649.0)
        ('chr18', 34446, 657.0)
        ('chrM',  308,   520.0)
        ('chrM',  309,   519.0)

    .. todo:: Optionally give a list of regions to walk, in that order.
    """
    # Todo: Do something with browser and track lines.
    # Todo: Better exceptions.
    # Todo: Prettify this code.
    # Todo: Detect if index does not agree with track.
    region = None

    idx, _ = index(track, force=force_index)

    if idx is None:
        regions = [None]
    else:
        # Todo: Sort in a way that is compatible with existing wiggle tracks.
        #     Inspiration could be sorted BAM files. GATK requires these to be
        #     sorted according to the order in the reference file.
        regions = sorted(r for r in idx if r != '_all')

    for expected_region in regions:

        if expected_region is not None:
            track.seek(idx[expected_region]['start'])

        state = create_state()

        for line in track:
            line_type, data = parse(line, state)
            if line_type == LineType.REGION:
                region = data
                if expected_region is not None and region != expected_region:
                    break
            elif line_type == LineType.DATA:
                for i in range(data.span):
                    yield region, data.position + i, data.value

        # Todo: If there is no index yet, but we read the whole file, write
        #     the index we just built. But if we go this route, I think we
        #     should somehow merge wiggle.walk with index.index since they
        #     will be doing virtually the same thing.
        #
        #     if expected_region is None:
        #        write_index(idx, track)


def zip_(*walkers):
    """
    Walk over all tracks simultaneously and for each position yield the
    region, position and a list of values for each track, or `None` in case
    the track has no value on the position.

    .. note:: This assumes the order of regions is compatible over all
        walkers. If you are unsure if this is the case for your input wiggle
        tracks, use the :func:`walk` function with the `force_index` keyword
        argument.

    :arg walkers: List of generators yielding tuples of (region, position,
        value) per defined position.
    :type walkers: list(generator(str, int, _))

    :return: Tuples of (region, position, values) per defined position.
    :rtype: generator(str, int, list(_))

    Example::

        >>> for x in zip_(walk(open('a.wig')), walk(open('b.wig'))):
        ...     x
        ...
        ('18', 7, [29.0, None])
        ('18', 8, [49.0, None])
        ('18', 9, [None, 87.0])
        ('MT', 1, [20.0, None])
        ('MT', 2, [36.0, 92.0])
    """
    # We work with a list of lookahead items. If a walker has no more items,
    # we use None in the lookahead list.
    items = []
    for walker in walkers:
        try:
            items.append(next(walker))
        except StopIteration:
            items.append(None)

    # Regions seen so far.
    regions = set()
    previous_region = None

    while True:
        # If all lookahead items are None, we are done.
        if not any(items):
            break

        # Get the next position to yield.
        region, position = min(item[0:2] for item in items if item is not None)

        # Check region order compatibility.
        if region != previous_region:
            if region in regions:
                raise Exception('The order of regions is not compatible')
            regions.add(region)
            previous_region = region

        # Yield all values at this position.
        values = [item[2] if item is not None and item[0:2] == (region, position)
                  else None for item in items]
        yield region, position, values

        # Advance the lookahead list where we just yielded a value.
        for i, item in enumerate(items):
            if item is not None and item[0:2] == (region, position):
                try:
                    items[i] = next(walkers[i])
                except StopIteration:
                    items[i] = None


def fill(walker, regions=None, filler=None):
    """
    Fill in undefined positions with `filler` (or `None`).

    :arg walker: Tuple of (region, position, value) per defined position.
    :type walker: generator(str, int, _)
    :arg regions: Dictionary with regions as keys and (start, stop) tuples as
        values. If not `None`, fill positions from start to stop (both
        including) in these regions. If `None`, fill positions in all
        regions between their first and last defined positions.
    :type regions: dict(str, (int, int))
    :arg filler: Value to use for filling undefined positions.
    :type filler: _

    :return: Tuples of (region, position, value) per position where value is
        `filler` if it was not defined in the original walker.
    :rtype: generator(str, int, _)

    Example::

        >>> for x in walk(open('a.wig')):
        ...     x
        ...
        ('MT', 3, 29.0)
        ('MT', 5, 49.0)
        ('MT', 8, 87.0)
        ('MT', 9, 20.0)
        >>> for x in fill(walk(open('a.wig')):
        ...     x
        ...
        ('MT', 3, 29.0)
        ('MT', 4, None)
        ('MT', 5, 49.0)
        ('MT', 6, None)
        ('MT', 7, None)
        ('MT', 8, 87.0)
        ('MT', 9, 20.0)

    .. note:: This might be a tiny bit memory-hungry on Python 2.x if there
        are *very* large gaps to fill since we use the range function to
        generate the positions. I don't think it's worth it to add version
        specific code paths for this.
    """
    previous_region = previous_position = None

    for region, position, value in walker:
        if region != previous_region:
            # Backlog.
            if regions is not None:
                try:
                    start, stop = regions[previous_region]
                    for p in range(max(previous_position + 1, start), stop + 1):
                        yield previous_region, p, filler
                except KeyError:
                    pass
            previous_region = region
            previous_position = None

        if regions is None:
            # No explicitely specified regions, fill everything.
            if previous_position is not None:
                for p in range(previous_position + 1, position):
                    yield region, p, filler
        else:
            try:
                # Specified where we must fill.
                start, stop = regions[region]
                if previous_position is None:
                    for p in range(start, min(position, stop + 1)):
                        yield region, p, filler
                else:
                    for p in range(max(previous_position + 1, start),
                                    min(position, stop + 1)):
                        yield region, p, filler
            except KeyError:
                # Region is not in explicitely specified regions, don't
                # fill it.
                pass

        previous_position = position
        yield region, position, value

    # Backlog.
    if regions is not None:
        try:
            start, stop = regions[previous_region]
            for p in range(max(previous_position + 1, start), stop + 1):
                yield previous_region, p, filler
        except KeyError:
            pass


def write(walker, track=sys.stdout, serializer=str, name=None,
          description=None):
    """
    Write items from a walker to a wiggle track.

    :arg walker: Tuples of (region, position, value) per defined position.
    :type walker: generator(str, int, _)
    :arg track: Writable file handle.
    :type track: file
    :arg serializer: Function making strings from values.
    :type serializer: function(_ -> str)
    :arg name: Optional track name (displayed to the left of the track in the
        UCSC Genome Browser).
    :type name: str
    :arg description: Optional track description (displayed as center label in
        the UCSC Genome Browser).
    :type description: str

    Example::

       >>> write(walk(open('a.wig')), name='My example')
       track type=wiggle_0 name="My example"
       variableStep chrom=1
       1 520.0
       4 536.0
       8 553.0
       variableStep chrom=MT
       1 568.0
       2 598.0
       6 616.0

    .. todo:: Options for variable or fixed step, window size, etc.
    """
    size = 0

    header = 'track type=wiggle_0'
    if name is not None:
        header += ' name="%s"' % name
    if description is not None:
        header += ' description="%s"' % description
    header += '\n'
    track.write(header)
    size += len(header)

    idx = {}
    current_region = None

    for region, position, value in walker:
        if region != current_region:
            line = 'variableStep chrom=%s\n' % region
            track.write(line)
            idx[region] = {
                'region': region,
                'start':  size,
                'stop':   size + len(line),
                'sum':    0,
                'count':  0}
            size += len(line)
            current_region = region
        line = '%d %s\n' % (position, serializer(value))
        track.write(line)
        idx[region]['stop'] = size + len(line)
        idx[region]['sum'] += value
        idx[region]['count'] += 1
        size += len(line)

    idx['_all'] = {
        'region': '_all',
        'start':  0,
        'stop':   size,
        'sum':    sum(r['sum'] for r in idx.values()),
        'count':  sum(r['count'] for r in idx.values())}

    write_index(idx, track)



"""
Merge any number of wiggle tracks in various ways.

The algorithm can be parameterized by a merge operation. Four of these
operations are predefined in :attr:`mergers`:

Merger ``sum``: Compute the sum of all values.

Merger ``mean``: Compute the mean of all values (and use 0 for undefined
values).

Merger ``count``: Compute the number of defined values.

Merger ``minus``: Subtract the second value from the first (and use 0 for
undefined values). Only defined on exactly two values.

.. moduleauthor:: Martijn Vermaat <martijn@vermaat.name>

.. Licensed under the MIT license, see the LICENSE file.
"""


from __future__ import division

from .wiggle import zip_


# Compute the sum of all values.
_merger_sum = lambda vs: sum(v for v in vs if v is not None)

# Compute the mean of all values (and use 0 for undefined values).
# Todo: Also provide a mean over only the defined values.
_merger_mean = lambda vs: _merger_sum(vs) / len(vs)

# Compute the number of defined values.
_merger_count = lambda vs: sum(1 for v in vs if v is not None)

# Substract the second from the first (and use 0 for undefined values).
_merger_minus = lambda vs: (vs[0] or 0) - (vs[1] or 0)


#: Predefined mergers. See :mod:`wiggelen.merge` for their definition.
mergers = {'sum':   _merger_sum,
           'mean':  _merger_mean,
           'count': _merger_count,
           'minus': _merger_minus}


def merge(*walkers, **options):
    """
    Merge wiggle tracks.

    This assumes the walkers have their regions in the same order. You can
    force this by using indices. Example::

        >>> from wiggelen import walk
        >>> walkers = [walk(open(track), force_index=True)
        ...            for track in ('a.wig', 'b.wig', 'c.wig')]
        >>> for x in merge(*walkers):
        ...     x
        ...
        ('18', 8, 849.0)
        ('18', 9, 987.0)
        ('MT', 1, 820.0)

    :arg walkers: List of generators yielding tuples of (region, position,
        value) per defined position.
    :type walkers: list(generator(str, int, _))
    :keyword merger: Merge operation (default: sum).
    :type merger: function(list(_) -> _)

    :return: Tuples of (region, position, merged value) per defined position
        in `walkers`.
    :rtype: generator(str, int, _)
    """
    # Todo: Would it be better to also pass region/position to the merger?
    merger = options.get('merger', mergers['sum'])

    for region, position, values in zip_(*walkers):
        yield region, position, merger(values)



import os
from setuptools import setup
import sys

if sys.version_info < (2, 6):
    raise Exception('Wiggelen requires Python 2.6 or higher.')

install_requires = []

# Python 2.6 does not include the argparse module.
try:
    import argparse
except ImportError:
    install_requires.append('argparse')

try:
    with open('README.rst') as readme:
        long_description = readme.read()
except IOError:
    long_description = 'See https://pypi.python.org/pypi/wiggelen'

# This is quite the hack, but we don't want to import our package from here
# since that's recipe for disaster (it might have some uninstalled
# dependencies, or we might import another already installed version).
distmeta = {}
for line in open(os.path.join('wiggelen', '__init__.py')):
    try:
        field, value = (x.strip() for x in line.split('='))
    except ValueError:
        continue
    if field == '__version_info__':
        value = value.strip('[]()')
        value = '.'.join(x.strip(' \'"') for x in value.split(','))
    else:
        value = value.strip('\'"')
    distmeta[field] = value

setup(
    name='wiggelen',
    version=distmeta['__version_info__'],
    description='Working with wiggle tracks in Python',
    long_description=long_description,
    author=distmeta['__author__'],
    author_email=distmeta['__contact__'],
    url=distmeta['__homepage__'],
    license='MIT License',
    platforms=['any'],
    packages=['wiggelen'],
    install_requires=install_requires,
    entry_points = {
        'console_scripts': ['wiggelen = wiggelen.commands:main']
        },
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Scientific/Engineering',
        ],
    keywords='bioinformatics'
)
